/*!
    \file projectile.h
  *  a projectile, that is been shooted by a weapon

    You can use this class to make some shoots, but this isn't the real idea. If you want to just test, if the
    shooting funcions work, use the Projectile class. But if you want to implement your own shoots its
    different:<br>
    Make a new class and derive it from Projectile. To have a weapon work well, reimplement the functions
    - void tick()
    - void draw()
    - void hit() (only if you have working collision detection)
    When you have implemented these functions you have just to add the projectiles to your weapon. You ll want
    to make this by looking into the function
    - Weapon::fire()
    there you just change the line:
    Projectile* pj = new Projectile();    TO     Projectile* pj = new MyOwnProjectileClass();
    and schwups it works... :)
*/

#ifndef _PROJECTILE_H
#define _PROJECTILE_H

#include "world_entity.h"
#include "vector.h"

class Vector;
class Weapon;
class ParticleEmitter;

class Projectile : public WorldEntity
{
  public:
    Projectile (Weapon* weapon);
    virtual ~Projectile ();

    void setFlightDirection(Quaternion flightDirection);
    void setVelocity(const Vector &velocity);
    void setLifeSpan(float lifeSpan);




    virtual void destroy ();

    virtual void tick (float time);
    virtual void draw ();

  protected:

    // energy
    float                 energyMin;
    float                 energyMax;


    float                 lifeCycle;                 //!< The percentage of the Lifetime done [0-1]
    float                 lifeSpan;                  //!< The entire lifespan of the Shoot.

    Vector                flightDirection;           //!< direction in which the shoot flighs
    Weapon*               weapon;                    //!< weapon the shoot belongs to.

    Vector                velocity;                  //!< velocity of the projectile.

    ParticleEmitter*      emitter;                   //!< For special effects each Projectile consists of an emitter.
};

#endif /* _PROJECTILE_H */
